#include <string.h>

#include "lamport.h"
#include "randombytes.h"
#include "shake256.h"

int crypto_ots_keypair(unsigned char *pk, unsigned char *sk) 
{
  int i;
  randombytes(sk,CRYPTO_OTS_SECRETKEYBYTES);
  for(i=0;i<16*CRYPTO_OTS_HASHBYTES;i++)
  {
    shake256(pk,CRYPTO_OTS_HASHBYTES,sk,CRYPTO_OTS_HASHBYTES);
    pk += CRYPTO_OTS_HASHBYTES;
    sk += CRYPTO_OTS_HASHBYTES;
  }
  return 0;
}

int crypto_ots_sign(unsigned char *sig,
                    const unsigned char *m,
                    unsigned long long mlen, 
                    const unsigned char *sk) 
{
  int i,j,bit;
  unsigned char h[CRYPTO_OTS_HASHBYTES];
  shake256(h,CRYPTO_OTS_HASHBYTES,m,mlen);
  for(i=0;i<CRYPTO_OTS_HASHBYTES;i++)
    for(j=0;j<8;j++)
    {
      bit = (h[i]>>j)&1;
      memcpy(sig,sk+bit*CRYPTO_OTS_HASHBYTES,CRYPTO_OTS_HASHBYTES);
      sig +=   CRYPTO_OTS_HASHBYTES;
      sk  += 2*CRYPTO_OTS_HASHBYTES;
    }
  return 0;
}

int crypto_ots_verify(const unsigned char *m,
                      unsigned long long mlen,
                      const unsigned char *sig,
                      const unsigned char *pk)
{
  int i,j,bit;
  unsigned char h[CRYPTO_OTS_HASHBYTES];
  unsigned char p[CRYPTO_OTS_HASHBYTES];

  shake256(h,CRYPTO_OTS_HASHBYTES,m,mlen);
  for(i=0;i<CRYPTO_OTS_HASHBYTES;i++)
  {
    for(j=0;j<8;j++)
    {
      bit = (h[i]>>j)&1;
      shake256(p,CRYPTO_OTS_HASHBYTES,sig,CRYPTO_OTS_HASHBYTES);
      if(memcmp(p,pk+bit*CRYPTO_OTS_HASHBYTES,CRYPTO_OTS_HASHBYTES)) return -1;
      sig +=   CRYPTO_OTS_HASHBYTES;
      pk  += 2*CRYPTO_OTS_HASHBYTES;
    }
  }
  return 0;
}
